import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.util.*;

/*
 * KMap.java 1.0 02/11/12
 * <br><br>
 * Author       : James McMahon
 * <br><br>
 * K-Map Object
 * <br><br>
 * Created      : 12/11/2001
 * <br>
 * Last Updated : 12/11/2001
 */

public class KMapType extends Vector{
	
	/**
	 * The vector containing the four elements of the top row.<br>
	 * These are four ints of either zero or one
	 */ 
	private Vector topRow;
	
	/**
	 * The vector containing the four elements of the bottom row.<br>
	 * These are four ints of either zero or one
	 */ 	
	private Vector bottomRow;
	
	/** 
	 * Gives a KMap randomally filled with zeros and ones
	 */
	public KMapType() {
		
		//get the random number generator
		Random randNum = new Random();		
		
		boolean allSame = true;
		
		//holds the temp number before storing in the Vectors 
		int number;
		
		//make sure we don't get a set of all zeros or ones
		while (allSame) {
		
			topRow    = new Vector(4);
			bottomRow = new Vector(4);

			//populate the top row
			for(int i=0;i<=3;i++) {
				number = randNum.nextInt(2);//Get a random int between 1 and 0
				topRow.addElement(new Integer(number));			//put it into the vector
			}
			
			//populate the bottom row
			for(int i=0;i<=3;i++) {
				number = randNum.nextInt(2);
				bottomRow.addElement(new Integer(number));
			}
			//returns true if all zero or all one
			allSame = checkSimilarity(topRow,bottomRow);
		}
		
		//create the KMap Type
		this.addElement(topRow);
		this.addElement(bottomRow);
	}
	
	/** 
	 * Creates a KMap with the specified top and bottom rows.
	 * <br><br>
	 * IMPORTANT - Make sure that the top and bottom rows contain 4 items apeice
	 * and they must be ints of either zero or one!!!
	 * <br><br>
	 * Used primarily for debugging purposes
	 *
	 * @param topRow A vector containing the four ints representing the top row's elements
	 * @param bottomRow A vector containing the four ints representing the bottom row's elements
	 */
	public KMapType(Vector topRow, Vector bottomRow) {
		this.addElement(topRow);
		this.addElement(bottomRow);
	}
	
	/** 
	 * Create a KMap with the top1 to bottom4 integers in the right places
	 * reading from left to right, top to bottom
	 * <br><br>
	 * Used primarily for debugging purposes
	 * <br>
	 * @param top1 The corresponding int at that location for the KMap that you want to create
	 * @param top2 The corresponding int at that location for the KMap that you want to create
	 * @param top3 The corresponding int at that location for the KMap that you want to create
	 * @param top4 The corresponding int at that location for the KMap that you want to create
	 * @param bottom1 The corresponding int at that location for the KMap that you want to create
	 * @param bottom2 The corresponding int at that location for the KMap that you want to create
	 * @param bottom3 The corresponding int at that location for the KMap that you want to create
	 * @param bottom4 The corresponding int at that location for the KMap that you want to create
	 */
	public KMapType(int top1, int top2, int top3, int top4, int bottom1, int bottom2, int bottom3, int bottom4){
	
		Vector top = new Vector();
		Vector bottom = new Vector();
		
		top.addElement(new Integer(top1));
		top.addElement(new Integer(top2));
		top.addElement(new Integer(top3));
		top.addElement(new Integer(top4));
		
		bottom.addElement(new Integer(bottom1));
		bottom.addElement(new Integer(bottom2));
		bottom.addElement(new Integer(bottom3));
		bottom.addElement(new Integer(bottom4));
				
		this.addElement(top);
		this.addElement(bottom);
	}	
	
	/** 
	 * Checks that topRow and BottomRow don't contain all 0's or all 1's
	 * 
	 * @param topRow The top row of the KMap
	 * @param bottomRow The bottom row of the KMap
	 *
	 * @return true if all same
	 * @return false if not all the same
	 */
	private boolean checkSimilarity(Vector topRow, Vector bottomRow) {
		
		boolean gotOne = false;
		boolean gotZero = false;
		
		Integer topValue;
		Integer bottomValue;
		
		//only need to loop until we find one 0 and one 1
		for(int count=0;count<=3;count++) {
				
			topValue    = (Integer) topRow.elementAt(count);
			bottomValue = (Integer) bottomRow.elementAt(count);
			
			//relatively inefficient but only got four items per vector so not important
			if (topValue.equals(new Integer(0))) { 
				gotZero = true;
			} else if (topValue.equals(new Integer(1))) { 
				gotOne = true;
			}
		
			if (bottomValue.equals(new Integer(0))) {
				gotZero = true;
			} else if (bottomValue.equals(new Integer(1))) { 
				gotOne = true;
			}
			
			//Minor improvement stops us doing unnecessary checks 
			//if we already know we've got a one and a zero
			if (gotOne && gotZero) {
				return false; //Got one of each so can't be all same
			}
		}
		//reached here so we've got a false and true situation
		return true;//they are similar - all zeros or all ones
	}
	
	/** 
	 * Returns the value, 0 or 1 that is held in location.
	 * <br>
	 * Location can be any value from zero to seven where the KMap is represented such that we can have 
	 * <br><br>0 1 2 3 <br> 
	 *         4 5 6 7<br><br>
	 * -1 is returned if the index is invalid.
	 * <br>
	 * @param location The location you want the value of 
	 *
	 * @return 0 If this is the value at location
	 * @return 1 If this is the value at location
	 * @return -1 If something is wrong (will also return other wierd values if the KMap has been incorrectly created)
	 */
	public int valueAt(int location) {

		Vector row = new Vector(1);

		if ((0 <= location) && (location <= 3)) {
			row = (Vector) this.getTopRow();
			return ((Integer) row.elementAt(location)).intValue();
		}
		else if ((4 <= location) && (location <= 7)) {
			row = (Vector) this.getBottomRow();
			return ((Integer) row.elementAt(location-4)).intValue();
		}
		else { 
			return -1;
		}
	}

	
	/** 
	 * Compares two KMaps to one another
	 * <br><br>
	 * Returns true if one == two
	 * 
	 * @return false If they don't match
	 * @return true If they do match
	 */
	public boolean equals(KMapType one, KMapType two) {
		return one.equals(two);		
	}
	
	/** 
	 * Combines kmap1 and kmap2 and returns the result of
	 * using an OR process. (i.e. 1 or 0 = 1, 1 or 1 = 1 , 0 or 0 = 0)
	 * 
	 * @param kMapTwo The second KMap to combine the first one to
	 * @return KMap The combined KMaps
	 */
	public KMapType combine(KMapType kMapTwo) {

		Vector KMap1Top    = new Vector(1);
		Vector KMap1Bottom = new Vector(1);
		Vector KMap2Top    = new Vector(1);
		Vector KMap2Bottom = new Vector(1);

		KMap1Top    = this.getTopRow();//the top row
		KMap1Bottom = this.getBottomRow();//the bottom row
	
		KMap2Top    = kMapTwo.getTopRow();//the top row
		KMap2Bottom = kMapTwo.getBottomRow();//the bottom row
		
		//loop through the length of each row and OR the values together
		for(int i=0;i<KMap1Top.size();i++) {
 
			//check and see if the adding KMap has a 1 in the top row
			if (KMap2Top.elementAt(i).equals(new Integer(1))) {
				//it does so set the value at i in the current kMap to 1
				KMap1Top.setElementAt(new Integer(1), i);
			}
			
			//check and see if the adding KMap has a 1 in the bottom row
			if (KMap2Bottom.elementAt(i).equals(new Integer(1))) {
				//it does so set the value at i in the current kMap to 1
				KMap1Bottom.setElementAt(new Integer(1), i);
			}
		}
		return new KMapType(KMap1Top,KMap1Bottom);//return a new KMap using the modified vectors of one
	}
	
	/**
	 * Tries to fit the instance into questionKMap by comparing to make sure a 1 in the instance is matched by
	 * a 1 in the questionMap. <br>
	 * This does not mean they match though, as the instance could have a zero where the questionKMap has a one. 
	 * 
	 * @param questionKMap The Kmap that you want to fit the instance into
	 * @return true If questionKMap does fit into this KMap (all 1's match i.e. no extra ones where there should be a zero)
	 * @return false If questionKMap doesn't fit into this KMap (a 1 is in the place of a zero in questionKMap so no fit)
	 */
	public boolean fitsInto(KMapType questionKMap) {
			
		Vector trialTop       = new Vector(1);
		Vector trialBottom    = new Vector(1);
		Vector questionTop    = new Vector(1);
		Vector questionBottom = new Vector(1);
		
		trialTop    = this.getTopRow();
		trialBottom = this.getBottomRow();

		questionTop = questionKMap.getTopRow();
		questionBottom = questionKMap.getBottomRow();
	
		//loop through every item in the trial vectors and compare anywhere
		//we find a one with the same location in the question vector
		//if we don't get a match with a one then return false - it doesn't fit
		for (int i=0; i<trialTop.size();i++) {
	
			//check the top row for a one
			if (trialTop.elementAt(i).equals(new Integer(1))) {
			//got a one so need to hope we have a one in the same location 
			//in the questionKMap
				if(!questionTop.elementAt(i).equals(new Integer(1))) {
					//not got a one so kick us out as it doesn't fit in
					return false;
				}
			}
			
			//check the bottom row for a one
			if (trialBottom.elementAt(i).equals(new Integer(1))) {
			//got a one so need to hope we have a one in the same location 
			//in the questionKMap
				if(!questionBottom.elementAt(i).equals(new Integer(1))) {
					//not got a one so kick us out as it doesn't fit in
					return false;
				}
			}			
		}
		//if we reach here then we've got a KMap that fits into the questionKMap
		//so return true
		return true;
	}
	
	/** 
	 * Returns the top row of the KMap in vector format
	 * @return Vector The top row of this KMap
	 */
	public Vector getTopRow() {	
		return (Vector) this.elementAt(0);
	}
	
	/** 
	 * Returns the bottom row of the KMap in vector format
	 * @return Vector The bottom row of this KMap
	 */
	public Vector getBottomRow() {
		return (Vector) this.elementAt(1);
	}	

	/** 
	 * Prints a neat representation of the KMap it is applied to as debug
	 */
	public void debugOutput() {
		
		Vector tR = this.getTopRow();
		Vector bR = this.getBottomRow();
		
		Integer tR1 = (Integer) tR.elementAt(0);
		Integer tR2 = (Integer) tR.elementAt(1);
		Integer tR3 = (Integer) tR.elementAt(2);
		Integer tR4 = (Integer) tR.elementAt(3);
		
		Integer bR1 = (Integer) bR.elementAt(0);
		Integer bR2 = (Integer) bR.elementAt(1);
		Integer bR3 = (Integer) bR.elementAt(2);
		Integer bR4 = (Integer) bR.elementAt(3);
		
		System.out.println(""+tR1+" "+tR2+" "+tR3+" "+tR4+" ");
		System.out.println(""+bR1+" "+bR2+" "+bR3+" "+bR4+" ");
	}
	
	/**
	 * Creates a toString of this KMap
	 * @return String A string representation of the KMap
	 */
	public String toString() {
		
		Vector tR = this.getTopRow();
		Vector bR = this.getBottomRow();
		
		Integer tR1 = (Integer) tR.elementAt(0);
		Integer tR2 = (Integer) tR.elementAt(1);
		Integer tR3 = (Integer) tR.elementAt(2);
		Integer tR4 = (Integer) tR.elementAt(3);
		
		Integer bR1 = (Integer) bR.elementAt(0);
		Integer bR2 = (Integer) bR.elementAt(1);
		Integer bR3 = (Integer) bR.elementAt(2);
		Integer bR4 = (Integer) bR.elementAt(3);
		
		return (""+tR1+" "+tR2+" "+tR3+" "+tR4+" \n"+bR1+" "+bR2+" "+bR3+" "+bR4+" ");
	}

}